/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlacc.c - deal with access types
 *
 * Jim McBeath, October 14, 1994
 */

#include "urlopen.h"

/* Find an access type, returns NULL if not found */
UACCESS *
UrlFindAccess(ctx,name)
UrlContext *ctx;
char *name;
{
	UACCESS *ut;

	if (!name || !*name) {
		UrlSetErrno(ctx,EPROTONOSUPPORT); /* protocol not supported */
		return (UACCESS *)0;	/* not found */
	}
	for (ut=ctx->accesslist; ut; ut=ut->next) {
		if (strcmp(name,ut->name)==0)
			return ut;
	}
	UrlSetErrno(ctx,EPROTONOSUPPORT);	/* protocol not supported */
	return (UACCESS *)0;	/* not found */
}

int	/* 0 if no error */
UrlRegister(ctx,access)
UrlContext *ctx;
UACCESS *access;
{
	UACCESS *ut;

	if (UrlFindAccess(ctx,access->name))
		return UrlSetErrno(ctx,EEXIST);	/* already exists */
	if (access->next)
		return UrlSetErrno(ctx,EMLINK);/* already linked to something */
	access->ctx = ctx;
	access->next = ctx->accesslist;
	ctx->accesslist = access;
	return 0;		/* OK */
}

UACCESS *	/* returns allocated struct, NULL on error */
UrlRegisterNew(ctx,name)
UrlContext *ctx;
char *name;	/* access name */
{
	UACCESS *access;

	if (UrlFindAccess(ctx,name)) {
		UrlSetErrno(ctx,EEXIST);	/* already exists */
		return (UACCESS *)0;
	}
	access = (UACCESS *)UrlMalloc(sizeof(UACCESS));
	if (!access) {
		UrlSetErrno(ctx,ENOMEM);
		return access;
	}
	memset(access,0,sizeof(UACCESS));
	access->name = UrlStrdup(name);
	if (!access->name) {
		UrlFree(access);
		UrlSetErrno(ctx,ENOMEM);
		return (UACCESS *)0;
	}
	UrlRegister(ctx,access);
	return access;	/* caller must fill in the fields now */
}

/* end */
