/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlcat.c - cat files from URL using the urlopen library
 *
 * Jim McBeath, October 14, 1994
 */

#include "urlopen.h"

extern char *strchr();

static void UrlCatPrintMessage();
extern char *UrlFtpFGetHtmlLine();
extern char *UrlGopherFGetHtmlLine();

typedef struct _urlcatproginfo {
	char *progname;
	UrlContext *uctx;
	int verbose;
	int raw;
	int ftp_dir;
	int gopher_menu;
	int getfile;
	int redirect_limit;
} UrlCatProgInfo;

int
UrlCat(argc,argv)
int argc;
char *argv[];
{
	char *url;
	int t;
	int mode=0;
	int i;
	char *url_base=0;
	UrlCatProgInfo proginfo;

	memset((char *)(&proginfo),'\0',sizeof(proginfo));
	proginfo.progname = argv[0];
	t = UrlInit(&proginfo.uctx);/* init our package with standard methods */
	if (t)
		return t;	/* TBD - need error info */
#define URL_REDIRECT_LIMIT 100
	proginfo.redirect_limit = URL_REDIRECT_LIMIT;
	UrlInitSigPipe();
	proginfo.uctx->appdata = (void *)(&proginfo);
	UrlSetMessageFunc(proginfo.uctx,UrlCatPrintMessage);
	if (argc<=1)
		return UrlCatUsage(&proginfo);
	for (i=1; i<argc; i++) {
		if (argv[i][0]=='-') {
			if (strcmp(argv[i],"-base")==0) {
				if (++i>=argc)
					return UrlCatUsage(&proginfo);
						/* missing arg for -base */
				url_base = argv[i];
			}
			else if (strcmp(argv[i],"-debug")==0)
				proginfo.uctx->debug = 1;
			else if (strcmp(argv[i],"-ftp_dir")==0)
				proginfo.ftp_dir = 1;
			else if (strcmp(argv[i],"-getfile")==0)
				proginfo.getfile = 1;
			else if (strcmp(argv[i],"-gopher_menu")==0)
				proginfo.gopher_menu = 1;
			else if (strcmp(argv[i],"-icon_prefix")==0) {
				if (++i>=argc)
					return UrlCatUsage(&proginfo);
						/* missing arg for -proxy */
				UrlSetIconPrefix(proginfo.uctx,argv[i]);
			}
			else if (strcmp(argv[i],"-meta")==0)
				mode = 'm';
			else if (strcmp(argv[i],"-parse")==0)
				mode = 'p';
			else if (strcmp(argv[i],"-passive")==0)
				proginfo.uctx->passive = 1;
			else if (strcmp(argv[i],"-print")==0)
				mode = 'u';
			else if (strcmp(argv[i],"-proxy")==0) {
				int l; char *s;
				if (++i>=argc)
					return UrlCatUsage(&proginfo);
						/* missing arg for -proxy */
				l = strlen(argv[i])+30;
				s = UrlMalloc(l);
				strcpy(s,"http://");
				if (!strchr(argv[i],':'))
					strcat(s,"localhost:");/* default host*/
				strcat(s,argv[i]); /* port number */
				UrlSetProxy(proginfo.uctx,s);
				UrlFree(s);
			}
			else if (strcmp(argv[i],"-raw")==0)
				proginfo.raw = 1;
#ifdef TERM
			else if (strcmp(argv[i],"-term")==0)
				proginfo.uctx->useterm = 1;
#endif
			else if (strcmp(argv[i],"-verbose")==0)
				proginfo.verbose = 1;
			else if (strcmp(argv[i],"-version")==0)
				return UrlCatVersion(&proginfo);
			else
				return UrlCatUsage(&proginfo);
		} else {
			if (url_base)
				url = UrlResolveRelative(proginfo.uctx,
					url_base,argv[i]);
			else
				url = argv[i];
			switch (mode) {
			case 'm':
				/* print meta-info */
				t = UrlCatPrintMeta(&proginfo,url);
				if (t)
					return t;
				break;
			case 'p':
				/* parse url and print pieces */
				t = UrlCatPrint(&proginfo,url);
				if (t)
					return t;
				break;
			case 'u':
				/* print url verbatim */
				printf("%s\n",url);
				break;
			default:
				t = UrlCatRead(&proginfo,url);
				if (t)
					return t;
				break;
			}
			url_base = url;		/* new base */
		}
	}
	return 0;
}

static void
UrlCatPrintMessage(ctx,type,str)
UrlContext *ctx;
int type;
char *str;
{
	UrlCatProgInfo *proginfo;

	proginfo = (UrlCatProgInfo *)(ctx->appdata);
	if (ctx->debug&&type==URL_MSG_DEBUG ||
	    proginfo->verbose&&type!=URL_MSG_DEBUG)
		fprintf(stderr,"%s\n",str);
}

int
UrlCatVersion(proginfo)
UrlCatProgInfo *proginfo;
{
	printf("%s\n%s\n",UrlVersion(),UrlCopyright());
	return 0;
}

int
UrlCatUsage(proginfo)
UrlCatProgInfo *proginfo;
{
	fprintf(stderr,"usage: %s [options] urllist\n",
		proginfo->progname);
	fprintf(stderr,"-base url  specify base for next url\n");
	fprintf(stderr,"-debug     turn on debugging\n");
	fprintf(stderr,"-ftp_dir   force parsing as ftp directory\n");
	fprintf(stderr,"-getfile   use FILE* rather than UFILE*\n");
	fprintf(stderr,"-gopher_menu   force parsing as gopher menu\n");
	fprintf(stderr,"-icon_prefix str     set prefix for internal icons\n");
	fprintf(stderr,"-meta      print meta-info about url\n");
	fprintf(stderr,"-parse     parse url and print fields\n");
	fprintf(stderr,"-passive   use passive mode for FTP connections\n");
	fprintf(stderr,"-print     just print url string (useful with -base)\n");
	fprintf(stderr,"-proxy [host:]port   specify proxy server\n");
	fprintf(stderr,"-raw       force raw output\n");
#ifdef TERM
	fprintf(stderr,"-term      use term to make connections\n");
#endif
	fprintf(stderr,"-verbose   print progress messages to stderr\n");
	fprintf(stderr,"-version   print urlopen verion to stdout and exit\n");
	return 2;
}

int
UrlCatPrintMeta(proginfo,url)
UrlCatProgInfo *proginfo;
char *url;
{
	UFILE *uf;
	char *s;

	uf = UrlOpen(proginfo->uctx,url,"r");
	if (!uf) {
		fprintf(stderr,"%s: %s\n",url,
			UrlGetErrorString(proginfo->uctx));
		return 1;
	}
	printf("Open OK on \"%s\"\n",url);
	s = UrlGetMeta(uf,"content-type");
	printf("Contents: %s\n",s?s:"<nil>");
	if (s) {
		s = UrlGetContentsCommand(proginfo->uctx,s);
		printf("Command: %s\n",s?s:"<nil>");
	}
	return 0;
}

int
UrlCatPrint(proginfo,url)
UrlCatProgInfo *proginfo;
char *url;
{
	UPARSE *up;

	up = UrlParse(proginfo->uctx,url);
	if (!up) {
		printf("No parse: error %d\n", UrlGetErrno(proginfo->uctx));
		return 1;
	}
	if (up->access) printf("access=%s\n",up->access);
	if (up->host)   printf("host=%s\n",  up->host);
	if (up->port)   printf("port=%s\n",  up->port);
	if (up->file)   printf("file=%s\n",  up->file);
	if (up->anchor) printf("anchor=%s\n",up->anchor);
	if (up->query)  printf("query=%s\n", up->query);
	return 0;
}

int
UrlCatRead(proginfo,url)
UrlCatProgInfo *proginfo;
char *url;
{
	UFILE *uf;
	int t, c;
	char *s;
	int state;

	uf = UrlOpen(proginfo->uctx,url,"r");
	if (!uf) {
		fprintf(stderr,"%s: %s\n",url,
			UrlGetErrorString(proginfo->uctx));
		return 1;
	}
	if (!proginfo->raw) {
		char *loc;
		loc = UrlGetMeta(uf,"location");	/* redirected? */
		if (loc) {
			if (proginfo->redirect_limit>0) {
				t = UrlCatRead(proginfo,loc);
				return t;
			} else {
				fprintf(stderr,"%s: too many redirects",url);
				return 1;
			}
		}
	}
	if (proginfo->ftp_dir) {
		state = 0;
		s = UrlFtpFGetHtmlLine(uf,&state);
		while (s) {
			fputs(s,stdout);
			s = UrlFtpFGetHtmlLine(uf,&state);
		}
	} else if (proginfo->gopher_menu) {
		state = 0;
		s = UrlGopherFGetHtmlLine(uf,&state);
		while (s) {
			fputs(s,stdout);
			s = UrlGopherFGetHtmlLine(uf,&state);
		}
	} else if (UrlIsFiltered(uf) && !proginfo->raw) {
		s = UrlGetLine(uf);
		while (s) {
			fputs(s,stdout);
			s = UrlGetLine(uf);
		}
	} else if (proginfo->getfile) {
		FILE *f;
		f = UrlGetFile(uf);
		if (!f) {
			perror("No FILE from UrlGetFile");
			UrlClose(uf);
			return 1;
		}
		while (!feof(f)) {
			c = fgetc(f);
			if (c==EOF) break;
			putchar(c);
		}
	} else {
		while (!UrlUeof(uf)) {
			c = UrlUgetc(uf);
			if (c==EOF) break;
			putchar(c);
		}
	}
	t = UrlClose(uf);
	if (t) {
		errno = UrlGetErrno(proginfo->uctx);
		perror("on close");
		return 1;
	}
	return 0;
}

#ifdef URL_CAT_MAIN
main(argc,argv)
int argc;
char *argv[];
{
	int t;
	t = UrlCat(argc,argv);
	exit(t);
}
#endif

/* end */
