/* Copyright 1995-1996 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlconn.c - socket connection stuff for url package
 *
 * Jim McBeath, October 18, 1994
 */

#include <sys/errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "urlopen.h"

#ifdef TERM
#include "client.h"
extern int termerrno;
#define TERMERRNO (termerrno?termerrno:-1)

extern char *term_server;

int
UrlConnectTermA(ctx,accessname,host,portnum)
UrlContext *ctx;
char *accessname;
char *host;
int portnum;
{
	int sock, t;

	UrlMessagef(ctx,URL_MSG_CONNECT,"Making term/%s connection to %s:%d",
		accessname, host, portnum);
	sock = connect_server(term_server);
	if (sock<0) {
		UrlSetErrno(ctx,TERMERRNO);
		return -1;
	}
	t = send_command(sock, C_PORT, 0, "%s:%d", host, portnum);
	if (t<0) {
		close(sock);
		UrlSetErrno(ctx,TERMERRNO);
		return -1;
	}
	send_command(sock, C_DUMB, 1, 0);
	return sock;
}

int
UrlConnectTerm(uf)
UFILE *uf;
{
	int portnum;
	UrlContext *ctx;

	ctx = uf->ctx;
	portnum = atoi(uf->parse->port);
	if (portnum<=0) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	return UrlConnectTermA(ctx,uf->access,uf->parse->port,portnum);
}

int	/* file descriptor */
UrlConnect4pTerm(uf,h0,h1,h2,h3,portnum)
UFILE *uf;
int h0,h1,h2,h3;
int portnum;
{
	int s,t;

	s = connect_server(0);
	if (s<0) {
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
	t = send_command(s, C_PORT, 0, "%d.%d.%d.%d:%d", h0,h1,h2,h3,portnum);
	if (t<0) {
		close(s);
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
	send_command(s, C_COMPRESS, 1, "n");
	send_command(s, C_DUMB, 1, 0);
	return s;
}

int
UrlListenTerm(uf,portstr)
UFILE *uf;
char *portstr;	/* RETURN port number string */
{
	int s;
	int t;

	s = connect_server(0);
	if (s<0) {
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
	t = send_command(s, C_BINDN, 0, 0, 0);
	if (t<0) {
		close(s);
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
#define MINPORTSTRLEN 24
	t = read(s,portstr,MINPORTSTRLEN);
	if (t<0) {
		close(s);
		UrlSetErrno(uf->ctx,errno);
		return -1;
	}
	if (t>=MINPORTSTRLEN)
		t = MINPORTSTRLEN-1;
	portstr[t]=0;
	return s;
}

int
UrlAcceptTerm(uf,ls)
UFILE *uf;
int ls;		/* the listen socket */
{
	char port[20];
	int s;
	int t;
	int pnum;

	t = read(ls,port,sizeof(port));
	if (t<0) {
		UrlSetErrno(uf->ctx,errno);
		return -1;	/* caller must close ls */
	}
	pnum = atoi(port);
	s = connect_server(0);
	if (s<0) {
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
	t = send_command(s, C_ACCEPT, 0, "%d", pnum);
	if (t<0) {
		close(s);
		UrlSetErrno(uf->ctx,TERMERRNO);
		return -1;
	}
	return s;
}
#endif

int
UrlConnectA(ctx,accessname,host,portnum)
UrlContext *ctx;
char *accessname;
char *host;
int portnum;
{
	int s, t, i;
	struct sockaddr_in sa;
	int sa_len;

	if (!host) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
#ifdef TERM
	if (ctx->useterm)
		return UrlConnectTermA(ctx,accessname,host,portnum);
#endif
	sa.sin_family = AF_INET;
	sa.sin_port = htons(portnum);
	if (isdigit(host[0])) {
		sa.sin_addr.s_addr = inet_addr(host);
	} else {
		struct hostent *he;
		he = gethostbyname(host);
		if (!he) {
			UrlSetErrno(ctx,URL_ENOHOST);
			return -1;
		}
		memcpy(&sa.sin_addr, he->h_addr, he->h_length);
	}
	s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s<0) {
		UrlSetErrno(ctx,errno);
		return -1;
	}
	UrlMessagef(ctx,URL_MSG_CONNECT,"Making %s connection to %s:%d",
		accessname, host, portnum);
	t = connect(s, (struct sockaddr *)(&sa), sizeof(sa));
	if (t<0) {
		UrlSetErrno(ctx,errno);
		close(s);
		return t;
	}
	sa_len = sizeof(sa);
	t = getsockname(s, (struct sockaddr *)(&sa), &sa_len);
	if (t<0) {
		UrlSetErrno(ctx,errno);
		close(s);
		return t;
	}
	for (i=0; i<4; i++) {
		ctx->ineta[i] = ((unsigned char *)(&sa.sin_addr))[i];
	}
	UrlMessagef(ctx,URL_MSG_DEBUG,"Our address on connect: %d,%d,%d,%d",
		ctx->ineta[0],ctx->ineta[1],ctx->ineta[2],ctx->ineta[3]);
	return s;
}

int		/* returns socket number, -1 on error */
UrlConnect(uf)
UFILE *uf;
{
	UrlContext *ctx;
	UPARSE *up;
	int portnum;

	ctx = uf->ctx;
	up = uf->parse;
	if (!up) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	if (!up->port) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	portnum = atoi(up->port);
	if (portnum<=0) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	return UrlConnectA(ctx,uf->access->name,up->host,portnum);
}

/* Open a socket and make a connection to a machine, given
 * it's 4 byte internet address and port number. */
int /* file descriptor */
UrlConnect4p(uf,h0,h1,h2,h3,portnum)
UFILE *uf;
int h0,h1,h2,h3;
int portnum;
{
	UrlContext *ctx;
	struct sockaddr_in soc_address;
	char *pa;
	int status;
	int s;
	int t;

	ctx = uf->ctx;
#ifdef TERM
	if (ctx->useterm) {
		return UrlConnect4pTerm(uf,h0,h1,h2,h3,portnum);
	}
#endif /* TERM */
	s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s<0) {
		UrlSetErrno(ctx,errno);
		return s;
	}

	pa = (char *)(&(soc_address.sin_addr));
	pa[0] = h0;	/* plug in the address we got back in from PASV */
	pa[1] = h1;
	pa[2] = h2;
	pa[3] = h3;
	soc_address.sin_port = htons(portnum);
	soc_address.sin_family = AF_INET;	    /* Family, host order  */

	t = connect(s, (struct sockaddr*)&soc_address,sizeof(soc_address));
	if (t<0){
		close(s);
		UrlSetErrno(ctx,errno);
		return t;
	}
	return s;
}

int
UrlListenPort(ctx,portnum)
UrlContext *ctx;
int portnum;
{
	int s, t;
	struct sockaddr_in sa;

	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = INADDR_ANY;
	sa.sin_port = htons(portnum);
	s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s<0) {
		UrlSetErrno(ctx,errno);
		return -1;
	}
	t = bind(s, (struct sockaddr *)(&sa), sizeof(sa));
	if (t<0) {
		UrlSetErrno(ctx,errno);
		close(s);
		return -1;
	}
	t = listen(s, 1);
	if (t<0) {
		UrlSetErrno(ctx,errno);
		close(s);
		return -1;
	}
	return s;
}

int	/* returns listen socket number, -1 on error */
UrlListen(uf,portstr)
UFILE *uf;
char *portstr;	/* RETURN port number string */
{
	UrlContext *ctx;
	UPARSE *up;
	int portnum;
	int s, t;
	struct sockaddr_in sa;
	int sa_len = sizeof(sa);

	ctx = uf->ctx;
	up = uf->parse;
	if (!up) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	if (!up->port) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
#ifdef TERM
	if (ctx->useterm)
		return UrlListenTerm(uf,portstr);
#endif
	portnum = atoi(up->port);
	if (portnum<=0) {
		UrlSetErrno(ctx,EINVAL);
		return -1;
	}
	s = UrlListenPort(ctx,0);
	if (s<0)
		return s;
	/* We presume that we first made a connection using UrlConnectA,
	 * which filled in ctx->ineta. */
	t = getsockname(s, (struct sockaddr *)(&sa), &sa_len); /* get port */
	if (t<0) {
		UrlSetErrno(ctx,errno);
		close(s);
		return t;
	}
	sprintf(portstr,"%d,%d,%d,%d,%d,%d",
		ctx->ineta[0], ctx->ineta[1],
		ctx->ineta[2], ctx->ineta[3],
		((unsigned char *)(&sa.sin_port))[0],
		((unsigned char *)(&sa.sin_port))[1]);
	return s;
}

int
UrlAcceptSocket(ctx,ls)
UrlContext *ctx;
int ls;
{
	int s;
	struct sockaddr_in sa;
	int sa_len = sizeof(sa);

	s = accept(ls,(struct sockaddr *)&sa,&sa_len);
	if (s<0) {
		UrlSetErrno(ctx,errno);
	}
	return s;
}

int
UrlAccept(uf,ls)
UFILE *uf;
int ls;		/* the listen socket */
{
	UrlContext *ctx;

	ctx = uf->ctx;
#ifdef TERM
	if (ctx->useterm)
		return UrlAcceptTerm(uf,ls);
#endif
	return UrlAcceptSocket(ctx,ls);
}

/* end */
