/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlcmd.c - stuff dealing with cmdents types
 *
 * Jim McBeath, October 27, 1994
 */

#include "urlopen.h"
#include <ctype.h>

extern char *getenv();
extern char *UrlFGetLine();

static UrlSymTab *UrlContCmds=0;

char *
UrlGetContentsCommand(ctx,cont)
UrlContext *ctx;
char *cont;		/* such as "image/jpeg" */
{
	return UrlSymFindSlashStar(ctx,ctx->contcmds,cont);
}

int	/* 0 if OK */
UrlSetContentsCommand(ctx,cont,cmd)
UrlContext *ctx;
char *cont;		/* such as "image/jpeg" */
char *cmd;		/* such as "xv %s" */
{
	return UrlSymSet(ctx,ctx->contcmds,cont,cmd);
}

int
UrlReadMailCap(ctx)
UrlContext *ctx;
{
#define PATHSIZE 1024
	char fn[PATHSIZE];
	char *home;
	FILE *fp;
	char *cont, *cmd;
	char *e;
	char *line;

	home = getenv("HOME");
	if (!home) home="";
	strcpy(fn,home);
	strcat(fn,"/");
	strcat(fn,".mailcap");
	fp = fopen(fn,"r");
	if (!fp) return 1;
	while (!feof(fp)) {
		line = UrlFGetLine(fp);
		if (!line || !*line || line[0]=='#')
			continue;
		while (isspace(*line)) line++;
		cont = line;
		while (*line && !isspace(*line) && *line!=';') line++;
		if (!*line) break;	/* no command given */
		*line = 0;
		line++;
		while (isspace(*line)) line++;
		if (!*line) break;
		cmd = line;
		e = line + strlen(line) - 1;
		if (*e=='\n') *e=0;
		UrlSetContentsCommand(ctx,cont,cmd);
	}
	fclose(fp);
}

int	/* 0 if OK */
UrlInitContCommands(ctx)
UrlContext *ctx;
{
	ctx->contcmds = UrlSymTabInit(ctx,"cont");
	if (!ctx->contcmds)
		return UrlGetErrno(ctx);

	/* set up standard mappings */
	UrlSetContentsCommand(ctx,"application/postscript","ghostview %s");

	UrlSetContentsCommand(ctx,"audio/basic","showaudio %s");

	UrlSetContentsCommand(ctx,"image/gif","xv %s");
	UrlSetContentsCommand(ctx,"image/jpeg","xv %s");
	UrlSetContentsCommand(ctx,"image/tiff","xv %s");
	UrlSetContentsCommand(ctx,"image/xwd","xwud -in %s");
	UrlSetContentsCommand(ctx,"image/*","xv %s");

	UrlSetContentsCommand(ctx,"text/html",":html");	/* internal */
	UrlSetContentsCommand(ctx,"text/plain",":html");	/* internal */

	UrlSetContentsCommand(ctx,"video/mpeg","mpeg_play %s");

	/* TBD - need to fill this out */
	UrlReadMailCap(ctx); /* read users $HOME/.mailcap for his extra stuff */

	return 0;
}

/* end */
