/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlerr.c - error routines for urlopen
 *
 * Jim McBeath, Oct 17, 1995
 */

#include "urlopen.h"
#include <errno.h>

extern int sys_nerr;

/* may be defined in stdio.h; if so, comment out here */
#if 1
extern const char *const sys_errlist[];
#endif

void
UrlClearErrno(ctx)
UrlContext *ctx;
{
	ctx->url_errno = 0;
}

int
UrlSetErrno(ctx,errnum)
UrlContext *ctx;
int errnum;
{
	ctx->url_errno = errnum;
	return errnum;
}

int
UrlAddErrno(ctx,errnum)
UrlContext *ctx;
int errnum;
{
	if (ctx->url_errno) {
		char *e;
		e = UrlGetErrorString(ctx);
		UrlSetErrorString(ctx,e);
	}
	ctx->url_errno = errnum;
	return errnum;
}

int
UrlGetErrno(ctx)
UrlContext *ctx;
{
	return ctx->url_errno;
}

char *
UrlGetErrnoStr(ctx,errnum)
UrlContext *ctx;
int errnum;
{
	int ee;
	static char buf[100];

	if (errnum>=0 && errnum<sys_nerr)
		return (char *)sys_errlist[errnum];
	switch (errnum) {
	case URL_ENOHOST: return "Unknown host";
	case URL_EPROTOFAIL: return "Protocol negotiation failed";
	default: break;
	}
	sprintf(buf,"<Error %d>",errnum);
	return buf;
}

char *
UrlGetErrorString(ctx)
UrlContext *ctx;
{
	char *ee, *es, *sep, *m;
	int l=0;

	if (ctx->url_errno) {
		ee = UrlGetErrnoStr(ctx,ctx->url_errno);
		l += strlen(ee);
	} else
		ee = "";
	if (ctx->errstr && ctx->errstr[0]) {
		es = ctx->errstr;
		l += strlen(es);
	} else
		es = "";
	if (*ee && *es) {
		sep = "\n";
		l += 1;
	} else
		sep = "";
	m = UrlMalloc(l+1);
	if (!m)
		return "No memory";
	sprintf(m,"%s%s%s",ee,sep,es);
	if (ctx->tstr)
		UrlFree(ctx->tstr);
	ctx->tstr = m;
	return m;
}

void
UrlSetErrorString(ctx,e)
UrlContext *ctx;
char *e;
{
	char *s;

	if (ctx->errstr) {
		UrlFree(ctx->errstr);
		ctx->errstr = (char *)0;
	}
	if (e==ctx->tstr) {
		ctx->errstr = ctx->tstr;
		ctx->tstr = (char *)0;
		return;
	}
	s = UrlStrdup(e);
	if (!s) {
		ctx->url_errno = ENOMEM;
		return;
	}
	ctx->url_errno = 0;
	ctx->errstr = s;
}

void
UrlAddErrorString(ctx,e)
UrlContext *ctx;
char *e;
{
	char *oe, *ne, *sep;
	int l;

	oe = UrlGetErrorString(ctx);
	l = strlen(oe)+strlen(e)+2;
	ne = UrlMalloc(l);
	if (!ne) {
		ctx->url_errno = ENOMEM;
		return;
	}
	if (*oe && *e)
		sep = "\n";
	else
		sep = "";
	sprintf(ne,"%s%s%s",oe,sep,e);
	if (ctx->errstr)
		UrlFree(ctx->errstr);
	ctx->errstr = ne;
	ctx->url_errno = 0;
}

/* end */
