/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlfile.c - file access
 *
 * Jim McBeath, March 12, 1995
 */
/* The "file" access type is just like FTP, except that it
 * checks for a hostname of "localhost", and if so, acts
 * just like localfile. */

#include "urlopen.h"

extern char *strrchr();

/* forward decls */
int UrlFileOpen();
int UrlFileClose();
char *UrlFileGetMeta();
int UrlFileIsLocalFile();

static
int
UrlFileIsLocalHost(uf)
UFILE *uf;
{
	if (!uf || !uf->parse || !uf->parse->host)
		return 1;	/* if all missing, assume local host */
	if (strcmp(uf->parse->host,"localhost")==0)
		return 1;
	else
		return 0;
}

void
UrlFileRegister(ctx)
UrlContext *ctx;
{
	UACCESS *access;

	access = UrlRegisterNew(ctx,"file");
	if (!access) return;
	access->open = UrlFileOpen;
	access->close = UrlFileClose;
	access->getmeta = UrlFileGetMeta;
	access->islocalfile = UrlFileIsLocalFile;
}

int		/* 0 if OK */
UrlFileOpen(uf)
UFILE *uf;
{
	if (UrlFileIsLocalHost(uf))
		return UrlLocalfileOpen(uf);
	else
		return UrlFtpOpen(uf);
}

int
UrlFileClose(uf)
UFILE *uf;
{
	if (UrlFileIsLocalHost(uf))
		return UrlLocalfileClose(uf);
	else
		return UrlFtpClose(uf);
}

int
UrlFileIsLocalFile(uf)
UFILE *uf;
{
	if (UrlFileIsLocalHost(uf))
		return UrlLocalfileIsLocalFile(uf);
	else
		return UrlFtpIsLocalFile(uf);
}

char *
UrlFileGetMeta(uf,name)
UFILE *uf;
char *name;
{
	extern char *UrlLocalfileGetMeta();
	extern char *UrlFtpGetMeta();

	if (strcmp(uf->parse->host,"localhost")==0)
		return UrlLocalfileGetMeta(uf);
	else
		return UrlFtpGetMeta(uf);
}

/* end */
