/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlftp.c - ftp access for url package
 *
 * Jim McBeath, October 16, 1994
 */

#include <varargs.h>
#include "urlopen.h"

extern char *getenv();
extern char *strchr();
extern char *strrchr();

/* forward decls */
int UrlFtpOpen();
int UrlFtpClose();
char *UrlFtpGetLine();
char *UrlFtpGetMeta();
int UrlFtpIsLocalFile();
int UrlFtpIsFiltered();
void UrlFtpFreeTdata();
static int UrlFtpSendGet();
static int UrlFtpCommand();
static int UrlFtpGetResponse();
static int UrlFtpSendCommand();
static int UrlFtpSendCommandString();
static char *UrlFtpGetFqhn();
static int UrlFtpInitTextBlock();
static int UrlFtpAppendTextBlock();

#define crlf "\r\n"

typedef struct _ftptextblock {
	char *text;	/* malloced buffer */
	int alloc;	/* size of text buffer */
	int count;	/* number of chars used in buffer */
} FtpTextBlock;

typedef struct _ftpinfo {
	int cfd;	/* fd for control connection */
	FILE *cfp;	/* FILE* version of cfd */
	int lfd;	/* fd for listen connection */
	int isdir;	/* true if we've got a directory */
	int state;	/* used during parsing of a dir listing */
	int status;	/* full status of last ftp command */
	FtpTextBlock response; /* last response text */
	FtpTextBlock conttext; /* all response continuation text */
	char *conttext_get;
	int conttext_char;
} FtpInfo;

void
UrlFtpRegister(ctx)
UrlContext *ctx;
{
	UACCESS *access;

	access = UrlRegisterNew(ctx,"ftp");
	if (!access) return;
	access->datasize = sizeof(FtpInfo);
	access->open = UrlFtpOpen;
	access->close = UrlFtpClose;
	access->getline = UrlFtpGetLine;
	access->getmeta = UrlFtpGetMeta;
	access->islocalfile = UrlFtpIsLocalFile;
	access->isfiltered = UrlFtpIsFiltered;
	access->tdatafree = UrlFtpFreeTdata;
	access->default_port = "21";
}

void
UrlFtpAddResponseError(uf)
UFILE *uf;
{
	UrlContext *ctx;
	FtpInfo *td;	/* our access-specific data */

	ctx = uf->ctx;
	td = (FtpInfo *)(uf->tdata);
	if (td->conttext.count>0)
		UrlAddErrorString(ctx,td->conttext.text);
	if (td->response.count>0) {
		char *p;
		p = strrchr(td->response.text,'\n');
		if (p)
			p++;
		else
			p = td->response.text;
		UrlAddErrorString(ctx,p);
	}
}

static
int	/* status */
UrlFtpUserPass(uf)
UFILE *uf;
{
	char *uname;
	int t;
	char *username, *hostname;
	UrlContext *ctx;

	ctx = uf->ctx;
	t = UrlFtpGetResponse(uf);
	if(t!=2) {
		UrlSetErrorString(ctx,"Failed to get expected FTP greeting code");
		UrlFtpAddResponseError(uf);
		return UrlAddErrno(ctx,URL_EPROTOFAIL);
	}
	UrlMessage(uf->ctx,URL_MSG_CONNECT,"Sending FTP login");
/* TBD - look for @user:pass at end of hostname in URL */
	uname = "anonymous";
	t = UrlFtpCommand(uf,"USER %s",uname);
	if (t!=3) {
		UrlSetErrorString(ctx,"USER command failed");
		UrlFtpAddResponseError(uf);
		return UrlAddErrno(ctx,URL_EPROTOFAIL);
	}
	username = getenv("USER");
	if (!username || !*username)
		username = "WWWuser";
	hostname = UrlFtpGetFqhn();
	t = UrlFtpCommand(uf,"PASS %s@%s",username,hostname);
	if (t==3) {
		t = UrlFtpCommand(uf,"ACCT noaccount");
	}
	if (t!=2) {
		UrlSetErrorString(ctx,"Login failed");
		UrlFtpAddResponseError(uf);
		return UrlAddErrno(ctx,URL_EPROTOFAIL);
	}
	UrlMessage(uf->ctx,URL_MSG_CONNECT,"FTP login successful");
	return 0;
}

int	/* status */
UrlFtpListen(uf)
UFILE *uf;
{
	UrlContext *ctx;
	FtpInfo *td;	/* our access-specific data */
	int t;
	char *s;
	char *fmt;
	char portstr[100];

	td = (FtpInfo *)(uf->tdata);
	ctx = uf->ctx;
	if (ctx->passive) {
		int h0,h1,h2,h3,p0,p1,portnum;
		t = UrlFtpCommand(uf,"PASV");
		if (t!=2) {
			UrlSetErrorString(ctx,"Ftp PASV command failed");
			UrlFtpAddResponseError(uf);
			return UrlAddErrno(ctx,URL_EPROTOFAIL);
		}
		s = td->response.text;
		if (strncmp(s,"227 Enter",9)==0)
			fmt = "227 Entering Passive Mode (%d,%d,%d,%d,%d,%d)";
		else
			fmt = "%d%d%d%d%d%d";
		t = sscanf(s,fmt,&h0,&h1,&h2,&h3,&p0,&p1);
		if (t<6) {
			UrlSetErrorString(ctx,"Bad response on PASV command");
			UrlFtpAddResponseError(uf);
			return UrlAddErrno(ctx,URL_EPROTOFAIL);
		}
		portnum = (p0<<8)+p1;
		t = UrlConnect4p(uf,h0,h1,h2,h3,portnum);
		if (t<0) {
			return UrlGetErrno(ctx);
		}
		uf->fd = t;
	} else {
		t = UrlListen(uf,portstr);
		if (t<0) {
			return UrlGetErrno(ctx);
		}
		td->lfd = t;
		t = UrlFtpCommand(uf,"PORT %s",portstr);
		if (t!=2) {
			UrlSetErrorString(ctx,"Ftp PORT command failed");
			UrlFtpAddResponseError(uf);
			return UrlAddErrno(ctx,URL_EPROTOFAIL);
		}
	}
	return 0;
}

int	/* status */
UrlFtpDataConn(uf)
UFILE *uf;
{
	UrlContext *ctx;
	int t;
	char *filename;
	FtpInfo *td;	/* our access-specific data */
	int isdir=0;

	ctx = uf->ctx;
	td = (FtpInfo *)(uf->tdata);
	t = UrlFtpCommand(uf,"TYPE I");
	if (t!=2) {
		UrlSetErrorString(ctx,"Ftp TYPE command failed");
		UrlFtpAddResponseError(uf);
		return UrlAddErrno(ctx,URL_EPROTOFAIL);
	}
	filename = uf->parse->file;
	if (!filename || !*filename)
		filename = "/";
	/* first, see if we can cd to it; if so, it's a directory */
	t = UrlFtpCommand(uf,"CWD %s",filename);
	if (t==2) {
		char *listcmd;
		listcmd = getenv("URLOPEN_FTP_LIST_CMD");
		if (!listcmd) listcmd="LIST";
		t = UrlFtpCommand(uf,listcmd);	/* get dir list */
		UrlParseMakeTrailingSlash(uf->parse);
		isdir = 1;
		/* success status for this is 1 */
	} else {
		/* cd failed, assume it's a file */
		int size=0;
		t = UrlFtpCommand(uf,"SIZE %s",filename); /* get file size */
		if (t!=5) {	/* 5xx is error */
			sscanf(td->response.text,"%*d %d",&size);
			if (size)
				uf->filesize = size;
		}
		t = UrlFtpCommand(uf,"RETR %s",filename);
	}
	if (t!=1) {
		UrlClearErrno(ctx);
		UrlFtpAddResponseError(uf);
		return UrlAddErrno(ctx,ENOENT);
	}
	if (!ctx->passive) {
		/* Get the data connection */
		t = UrlAccept(uf,td->lfd);
		if (t<0)
			return UrlGetErrno(ctx);
		uf->fd = t;
	}
	td->isdir = isdir;
	return 0;
}

int	/* status */
UrlFtpOpen(uf)
UFILE *uf;
{
	int t;
	FtpInfo *td;	/* our access-specific data */
	UrlContext *ctx;

	ctx = uf->ctx;
	td = (FtpInfo *)(uf->tdata);
	td->cfd = -1;
	td->lfd = -1;
	if (!uf->parse->port)
		uf->parse->port = "21";	/* default port as string */
	td->cfd = UrlConnect(uf);		/* make the connection */
	if (td->cfd<0)
		return UrlGetErrno(ctx);
	td->cfp = fdopen(td->cfd,"r+");
	if (!td->cfp) {
		UrlSetErrno(ctx,errno);
		goto err;
	}
	t = UrlFtpUserPass(uf);
	if (t)
		goto err;
	t = UrlFtpListen(uf);
	if (t)
		goto err;
	t = UrlFtpDataConn(uf);
	if (t)
		goto err;
	/* so I guess we're all ready to read the file now... */
	return 0;
err:
	UrlFtpClose(uf);
	return UrlGetErrno(ctx);
}

static
int	/* FTP status */
UrlFtpCommand(uf,fmt,va_alist)
UFILE *uf;
char *fmt;
va_dcl
{
	va_list pvar;
	static char buf[1200];
	int t;

	va_start(pvar);
	vsprintf(buf,fmt,pvar);
	strcat(buf,crlf);
	t = UrlFtpSendCommandString(uf,buf);
	va_end(pvar);
	if (t)
		return -1;	/* error sending command */
	t = UrlFtpGetResponse(uf);
	return t;
}

static
int
UrlFtpSendCommand(uf,fmt,va_alist)
UFILE *uf;
char *fmt;
va_dcl
{
	va_list pvar;
	static char buf[1200];
	int t;

	va_start(pvar);
	vsprintf(buf,fmt,pvar);
	strcat(buf,crlf);
	t = UrlFtpSendCommandString(uf,buf);
	va_end(pvar);
	return t;
}

static
int	/* status (errno) */
UrlFtpSendCommandString(uf,str)
UFILE *uf;
char *str;
{
	FtpInfo *td;
	int t;

	td = (FtpInfo *)(uf->tdata);
	if (uf->ctx->debug)
		UrlMessagef(uf->ctx,URL_MSG_DEBUG,"Send to FTP: %s",str);
	t = UrlFWrite(td->cfd,str,strlen(str));
	if (t<0)
		return UrlSetErrno(uf->ctx,errno);
	return 0;	/* command successfully sent */
}

/* Reads FTP reply including continuation lines.
 * Returns status in reply, or <0 for other errors.
 */
static
int	/* ftp reply status divided by 100, -1 on other error w errno set */
UrlFtpGetResponse(uf)
UFILE *uf;
{
	FILE *f;
	char *l;
	int status;
	char contchar;
	FtpInfo *td;
	int n, llen;
	int t;

	td = (FtpInfo *)(uf->tdata);
	f = td->cfp;
	t = UrlFtpInitTextBlock(uf,&(td->response));
	if (t) return t;
	if (!td->conttext.text) {
		t = UrlFtpInitTextBlock(uf,&(td->conttext));
		if (t) return t;
	}
	do {
		l = UrlFGetLineNoNl(f);
		if (uf->ctx->debug)
			UrlMessagef(uf->ctx,URL_MSG_DEBUG,
				"FTP response line: %s", l);
		if (l[0]==((char)EOF)) {
			/* TBD - error? */
		}
		UrlFtpAppendTextBlock(uf,&(td->response),l);
		sscanf(l,"%d%c", &status, &contchar);
		if (contchar=='-') {
			/* store up all continued lines for output later */
			UrlFtpAppendTextBlock(uf,&(td->conttext),l);
		}
	} while (contchar=='-');
	if (status==421) {
		; /* TBD - need to close socket? */
	}
	td->status = status;
	return status/100;
}

static
int
UrlFtpInitTextBlock(uf,tbp)
UFILE *uf;
FtpTextBlock *tbp;
{
	if (!tbp->text) {
#define INITIAL_RBUF_SIZE 1000
		tbp->text = UrlMalloc(INITIAL_RBUF_SIZE);
		if (!tbp->text)
			return -1;
		tbp->alloc = INITIAL_RBUF_SIZE;
	}
	tbp->text[0] = 0;
	tbp->count = 0;
	return 0;
}

static
int
UrlFtpAppendTextBlock(uf,tbp,l)
UFILE *uf;
FtpTextBlock *tbp;
char *l;
{
	int llen, n;

	llen = strlen(l);
	n = tbp->count + llen + 2;
	if (n>tbp->alloc) {
		char *new_buf;
		int new_alloc = 2*tbp->alloc;
		while (new_alloc<n)
			new_alloc *= 2;
		new_buf = UrlRealloc(tbp->text,new_alloc);
		if (!new_buf)
			return -1;
		tbp->text = new_buf;
		tbp->alloc = new_alloc;
	}
	if (tbp->count>0) {
		strcpy(tbp->text+tbp->count,"\n");
		tbp->count++;
	}
	strcpy(tbp->text+tbp->count,l);
	tbp->count += llen;
	return 0;
}

int	/* status */
UrlFtpClose(uf)
UFILE *uf;
{
	FtpInfo *td;	/* our access-specific data */
	int t;

/* We set t a bunch of times; as status, we'll return whatever the
 * status of the last close call was. */
	t = 0;
	td = (FtpInfo *)(uf->tdata);
	if (td->lfd>=0) {
		t = close(td->lfd);
		td->lfd = -1;
	}
	if (td->cfp) {
		t = fclose(td->cfp);
		td->cfp = (FILE *)0;
		td->cfd = -1;
	} else if (td->cfd>=0) {
		t = close(td->cfd);
		td->cfd = -1;
	}
	if (uf->fd>=0) {
		t = close(uf->fd);
		uf->fd = -1;
	}
	return t;
}

char *
UrlFtpGetMeta(uf,name)
UFILE *uf;
char *name;
{
	UrlContext *ctx;
	FtpInfo *td;

	td = (FtpInfo *)(uf->tdata);
	ctx = uf->ctx;
	if (strcmp(name,"content-type")==0) {
		UPARSE *parse;
		char *ext, *ct;

		if (td->isdir)
			return "text/html";  /* directories filtered to HTML */
		parse = uf->parse;
		if (!parse->file)
			return "";	/* no type specified */
		ext = strrchr(parse->file,'.');
		if (!ext)
			return "";
		ct = UrlGetExtContType(ctx,ext+1);
		return ct;
	}
	UrlSetErrno(uf->ctx,EINVAL);
	return (char *)0;
}

int
UrlFtpIsFiltered(uf)
UFILE *uf;
{
	FtpInfo *td;

	td = (FtpInfo *)(uf->tdata);
	return td->isdir;
}

int			/* number of fields found */
UrlFtpSplitFields(s,pp,np)
char *s;		/* the string to split - puts nulls into it in place */
char **pp;		/* array to fill with pointers */
int np;			/* size of pp array */
{
	return UrlSplitFields(s,' ',1,pp,np);
}

/* Parse the line coming back from the directory listing */
void
UrlFtpParseDirLine(rawline,fileinfo)
char *rawline;
UrlDirFileInfo *fileinfo;
{
#define FTPMAXDIRFIELDS 20
	char *fields[FTPMAXDIRFIELDS];
	char *prot;
	int n;
#define FTPDATEBUFSIZE 100
	static char datebuf[FTPDATEBUFSIZE];

	memset((char *)fileinfo,'\0',sizeof(UrlDirFileInfo));
	if (!rawline)
		return;
/* TBD - probably need to do some didling to see what format it is */
	n = UrlFtpSplitFields(rawline,fields,FTPMAXDIRFIELDS);
	if (n<8)
		return;	/* bad format */
/* Unix format:
  fields[0] = protection
	[1] = number of links
	[2] = uid
	[3] = gid
	[4] = size
	[5,6,7] = date
	[8] = filename
	[9] = "->" if symlink
	[10] = target of symlink
 */
	prot = fields[0];
	if (prot[0]=='d')
		fileinfo->isdir = 1;
	else if (prot[0]=='l')
		;	/* TBD - symlink */
	else if (prot[3]=='x' || prot[6]=='x' || prot[9]=='x' ||
	    prot[3]=='t' || prot[6]=='t' || prot[9]=='t' ||
	    prot[3]=='s' || prot[6]=='s' || prot[9]=='s')
		fileinfo->isexec = 1;
	fileinfo->size = atoi(fields[4]);
	fileinfo->name = fields[8];
	if (n>10)
		fileinfo->symlink = fields[10];
	/* TBD - check length of these fields to ensure no datebuf overflow */
	sprintf(datebuf,"%s %2s %5s",fields[5],fields[6],fields[7]);
	fileinfo->date = datebuf;
}

/* Parse a file and return HTML.
 * Separated from UrlFtpGetLine so that url_cat can open a local file
 * and pass it through this filter. */
char *
UrlFtpFGetHtmlLine(uf,statep)
UFILE *uf;
int *statep;
{
	char *line, *s, *e;
	UrlDirFileInfo fileinfo;
#define FTPHTMLBUFSIZE 1200
	static char buf[FTPHTMLBUFSIZE];
	FtpInfo *td;

	td = (FtpInfo *)(uf->tdata);
	if (uf->fd<0)
		return (char *)0;
	/* We're scanning a directory, so we need to parse the line
	 * and turn it into HTML */
	switch (*statep) {
	case 0:	/* need to read dir header, return HTML header */
		/* TBD - build a nicer title than this */
		*statep = 2;	/* continue header on next call */
		s = UrlUnparseC(uf->parse);
		sprintf(buf,
		   "<HTML><HEADER><TITLE>FTP Listing for %s</TITLE></HEADER>\n",
			s);
		if (td->conttext.count>0)
			td->conttext_get = td->conttext.text;
		else
			td->conttext_get = (char *)0;
		td->conttext_char = 0;
		return buf;
	case 2:
		(*statep)++;
		s = UrlUnparseC(uf->parse);
		sprintf(buf,"<BODY><H1>FTP Listing for %s</H1>\n",s);
		return buf;
	case 3:
		(*statep)++;
		if (td->conttext_get)
			s = "<HR><PRE>\n";
		else {
			(*statep)++;	/* skip conttext step */
			s = "<PRE>\n";
		}
		return s;
	case 4:	/* add continuation text lines */
		s = td->conttext_get;
		if (!s) {
			(*statep)++;	/* time to move on */
			return "</PRE><HR><PRE>\n";
		}
		if (td->conttext_char)
			*s = td->conttext_char;
		e = strchr(s,'\n');
		if (!e) {
			/* this is the last text line */
			td->conttext_get = 0;
			return s;
		}
		e++;	/* point to char after newline */
		td->conttext_char = *e;	/* save it */
		*e = 0;	/* clear it */
		td->conttext_get = e;
		return s;
	case 5:
		if (!uf->parse->file || !uf->parse->file[0] ||
		    strcmp(uf->parse->file,"/")==0) {
			/* This is root dir, don't add parent line */
			*statep = 1;	/* next time, start on file stuff */
		} else
			(*statep)++;	/* put in parent line */
		s = UrlUGetLine(uf);
		return s;		/* print the directory sum */
	case 6:
		/* add a "parent" line */
		memset((char *)(&fileinfo),0,sizeof(fileinfo));
		fileinfo.name = "Parent Directory";
		fileinfo.symlink = "..";
		fileinfo.date = "";
		fileinfo.isdir = 1;
		s = UrlDirFileHtml(uf,&fileinfo);
		*statep = 1;	/* next time, start on file stuff */
		return s;
	case 1: /* need to read dir line about file, return HTML for it */
		line = UrlUGetLineNoNl(uf);
		if (uf->ctx->debug)
			UrlMessagef(uf->ctx,URL_MSG_DEBUG,
				"FTP data line: %s", line?line:"(nil)");
		UrlFtpParseDirLine(line,&fileinfo);
		s = UrlDirFileHtml(uf,&fileinfo);
		if (s)
			return s;
		if (line && *line)
			return "(Bad line)\n";
		*statep = -1;	/* done */
		/* finish it up */
		return "</PRE></BODY></HTML>\n";
		break;
	default:
		errno = 0;	/* end of file */
		return (char *)0;	/* no more */
	}
}

char *
UrlFtpGetLine(uf)
UFILE *uf;
{
	char *rawline;
	FtpInfo *td;

	td = (FtpInfo *)(uf->tdata);
	if (!td->isdir) {
		rawline = UrlUGetLine(uf);
		return rawline;
	}
	return UrlFtpFGetHtmlLine(uf,&(td->state));
}

int
UrlFtpIsLocalFile(uf)
UFILE *uf;
{
	return 0;	/* at this point, nothing is local */
}

void
UrlFtpFreeTdata(uf)
UFILE *uf;
{
	FtpInfo *td;

	td = (FtpInfo *)(uf->tdata);
	if (td->response.text) {
		UrlFree(td->response.text);
		td->response.text = 0;
		td->response.alloc = td->response.count = 0;
	}
}

/* Get fully qualified host name */
#define DOMAINNAME_SIZE 512
#define HOSTNAME_SIZE 128
static char fqhn[DOMAINNAME_SIZE+HOSTNAME_SIZE+2]={0};
static
char *
UrlFtpGetFqhn()
{
	char domainname[DOMAINNAME_SIZE+1];
	char hostname[HOSTNAME_SIZE+1];

	if (fqhn[0])
		return fqhn;
	getdomainname(domainname,sizeof(domainname)-1);
	domainname[sizeof(domainname)-1] = 0;
	gethostname(hostname,sizeof(hostname)-1);
	hostname[sizeof(hostname)-1] = 0;
	sprintf(fqhn,"%s.%s",hostname,domainname);
	return fqhn;
}

/* end */
