/* Copyright 1995-1996 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlopen.h - include file for UrlOpen and related functions.
 *
 * Jim McBeath, October 14, 1994
 */

#include <stdio.h>
#include <errno.h>

#ifdef __STDC__	/* use prototypes when available */
#define UARGS(args) args
#else
#define UARGS(args) ()
#endif

/* Definition of context - where we keep "global" info */
typedef struct _urlcontext {
	int useterm;
	int passive;
	int debug;
	int url_errno;
	int is_proxy;	/* true if we are a proxy server */
	char *errstr;
	char *tstr;
	char *icon_prefix;
	unsigned char ineta[4];	/* our inet addr, filled in by UrlConnectA */
	struct _uaccess *accesslist;
	struct _urlsymtab *contcmds;
	struct _urlsymtab *extmap;
	struct _urlsymtab *cont_icon_map;
	struct _ufile *ufilefreelist;
	struct _uparse *parsefreelist;
	struct _uparse *use_proxy; /* if set, points to parse info for proxy */
				/* This is if we are client to a proxy */
	void (*msgfunc)();
	int (*prereadfunc)();
	void *appdata;	/* application can set this to whatever */
} UrlContext;

/* Definition of an access type (e.g. "ftp:") */
typedef struct _uaccess {
	char *name;		/* method name as it appears in a url */
	struct _uaccess *next;
	struct _urlcontext *ctx;
	int datasize;		/* bytes to malloc for tdata in UFILE */
	char *default_port;
	int (*open)();
	int (*close)();
	int (*getdesc)();
	char *(*getline)();
	char *(*getmeta)();
	int (*islocalfile)();
	int (*isfiltered)();
	void (*tdatafree)();	/* func to free contents of tdata in UFILE */
} UACCESS;

typedef struct _uparse {
	struct _uparse *next;	/* for freelist */
	struct _urlcontext *ctx;
	char *url;	/* example: http://foo.bar:123/a/b#c?d */
	char *unparsed;	/* filled in by UrlUnparse */
	char *access;	/* example: http */
	char *host;		/*  foo.bar */
	char *port;		/*  123 */
	char *file;		/*  a/b */
	char *anchor;		/*  c */
	char *query;		/*  d */
	char *f1;	/* free this when freeing struct */
	char *f2;	/* free this as well as url when freeing struct */
} UPARSE;

typedef struct _ufile {
	UACCESS *access;	/* access type */
	struct _ufile *next;	/* for freelist chain */
	struct _urlcontext *ctx;
	UPARSE *parse;
	char *mode;	/* requested open mode */
	int xccount;	/* count of chars discarded by NoNl routine */
	int filesize;	/* number of bytes in the file (0 if unknown) */
	int bytesread;	/* number of bytes we have read so far */
	int fd;		/* input descriptor used to fill ibuf */
	int ibuf_alloc;
	unsigned char *ibuf;	/* here's where we read our input */
	unsigned char *ibuf_get;	/* points to next valid char */
	unsigned char *ibuf_end;	/* one past last valid char */
	int eofflag;	/* set when we get eof on fd */
	void *tdata;	/* access-specific data */
} UFILE;

/* Symbol table stuff */
typedef struct _urlsym {
	struct _urlsym *next;
	char *name;
	char *value;
} UrlSym;

typedef struct _urlsymtab {
	char *name;	/* name of symbol table */
	struct _urlcontext *ctx;
	UrlSym *first;	/* first sym in chain */
} UrlSymTab;

/* We use this structure when parsing a directory listing line */
typedef struct _urldirfileinfo {
	short isdir;	/* true if file is a directory (from stat) */
	short isexec;	/* true if file is executable (from stat) */
	int size;
	char *date;
	char *name;
	char *symlink;	/* target of symlink (if any) */
} UrlDirFileInfo;

/* types for messages passed to message callback */
#define URL_MSG_CONNECT 1	/* message during connection */
#define URL_MSG_READING 2	/* progress message when reading */
#define URL_MSG_DEBUG   3	/* debugging messages */

/* Our error codes, starting at 2000 */
#define URL_ENOHOST 2001	/* unknown host */
#define URL_EPROTOFAIL 2002	/* protocol negotiation failed */

int        UrlAccept UARGS((UFILE *uf, int ls));
int        UrlAcceptSocket UARGS((UrlContext *ctx, int ls));
int        UrlAddErrno UARGS((UrlContext *ctx, int errnum));
void       UrlAddErrorString UARGS((UrlContext *ctx, char *s));
void       UrlClearErrno UARGS((UrlContext *ctx));
char      *UrlCopyright UARGS((void));
int        UrlClose UARGS((UFILE *uf));
char      *UrlDirFileHtml UARGS((UFILE *uf, UrlDirFileInfo *fileinfo));
char      *UrlErrorString UARGS((int errnum));
char      *UrlFGetLine UARGS((FILE *f));
char      *UrlFGetLineNoNl UARGS((FILE *f));
UACCESS   *UrlFindAccess UARGS((UrlContext *ctx, char *name));
void       UrlFree UARGS((void *p));
void       UrlFreeUfile UARGS((UFILE *uf));
char      *UrlGetContentsCommand UARGS((UrlContext *ctx, char *conttype));
char      *UrlGetContIconType UARGS((UrlContext *ctx, char *cont));
char      *UrlGetErrorString UARGS((UrlContext *ctx));
int        UrlGetErrno UARGS((UrlContext *ctx));
char      *UrlGetExtContType UARGS((UrlContext *ctx, char *ext));
FILE      *UrlGetFile UARGS((UFILE *uf));
char      *UrlGetFileString UARGS((UFILE *uf));
char      *UrlGetLine UARGS((UFILE *uf));
FILE      *UrlGetLocalFile UARGS((UFILE *uf));
char      *UrlGetMeta UARGS((UFILE *uf, char *name));
int        UrlInit UARGS((UrlContext **rctx));
int        UrlListen UARGS((UFILE *uf, char *portstr));
int        UrlListenPort UARGS((UrlContext *ctx, int portnum));
void      *UrlMalloc UARGS((unsigned int size));
void       UrlMessage UARGS((UrlContext *ctx, int type, char *msg));
#if 0	/* gcc complains about argument mismatch, so forget it */
void       UrlMessagef UARGS((UrlContext *ctx, int type, char *fmt, ...));
#endif
int        UrlNewContext UARGS((UrlContext **rctx));
UPARSE    *UrlNewParse UARGS((UrlContext *ctx));
UFILE     *UrlNewUfile UARGS((UACCESS *access));
UPARSE    *UrlParse UARGS((UrlContext *ctx, char *url));
void       UrlParseMakeTrailingSlash UARGS((UPARSE *parse));
UFILE     *UrlOpen UARGS((UrlContext *ctx, char *url, char *mode));
void      *UrlRealloc UARGS((void *p, unsigned int size));
int        UrlRegister UARGS((UrlContext *ctx, UACCESS *access));
UACCESS   *UrlRegisterNew UARGS((UrlContext *ctx, char *name));
char      *UrlResolveRelative UARGS((UrlContext *ctx,
			char *base, char *partial));
char      *UrlResolveRelativeP UARGS((UrlContext *ctx,
			UPARSE *base, char *partial));
void       UrlSetErrorString UARGS((UrlContext *ctx, char *s));
int        UrlSetErrno UARGS((UrlContext *ctx, int errnum));
void       UrlSetIconPrefix UARGS((UrlContext *ctx, char *icon_prefix));
int        UrlSetProxy UARGS((UrlContext *ctx, char *proxy));
char      *UrlSymFind UARGS((UrlContext *ctx, UrlSymTab *table, char *name));
char      *UrlSymFindSlashStar UARGS((UrlContext *ctx, UrlSymTab *table,
				char *name));
int        UrlSymSet UARGS((UrlContext *ctx, UrlSymTab *table,
				char *name, char *value));
void       UrlSymTabFree UARGS((UrlContext *ctx, UrlSymTab *st));
UrlSymTab *UrlSymTabInit UARGS((UrlContext *ctx, char *name));
void       UrlShowBytesRead UARGS((UFILE *uf));
char      *UrlStrdup UARGS((char *s));
int        UrlUBufferCount UARGS((UFILE *uf));
int        UrlUeof UARGS((UFILE *uf));
int        UrlUfileno UARGS((UFILE *uf));
int        UrlUgetc UARGS((UFILE *uf));
char      *UrlUGetLine UARGS((UFILE *uf));
char      *UrlUGetLineNoNl UARGS((UFILE *uf));
char      *UrlUgets UARGS((char *buf, int size, UFILE *uf));
char      *UrlUnparse UARGS((UPARSE *parse));
char      *UrlUnparseC UARGS((UPARSE *parse));
int        UrlUread UARGS((char *ptr, int size, int nel, UFILE *uf));
char      *UrlVersion UARGS((void));

/* end */
