/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlparse.c - parse out various parts of a url spec
 *
 * Jim McBeath, October 16, 1994
 */

#include "urlopen.h"

extern char *strchr();

UPARSE *
UrlNewParse(ctx)
UrlContext *ctx;
{
	UPARSE *up;

	if (!ctx->parsefreelist) {
		/* TBD - make this allocate more than one at a time? */
		ctx->parsefreelist = (UPARSE *)UrlMalloc(sizeof(UPARSE));
		if (!ctx->parsefreelist) {
			UrlSetErrno(ctx,ENOMEM);
			return (UPARSE *)0;
		}
		memset(ctx->parsefreelist,0,sizeof(UPARSE));
	}
	up = ctx->parsefreelist;
	up->ctx = ctx;
	ctx->parsefreelist = up->next;
	up->next = 0;
	return up;
}

void
UrlFreeParse(up)
UPARSE *up;
{
	UrlContext *ctx;

	ctx = up->ctx;
	if (up->url)
		UrlFree(up->url);
	if (up->f1)
		UrlFree(up->f1);
	if (up->f2)
		UrlFree(up->f2);
	if (up->unparsed)
		UrlFree(up->unparsed);
	memset(up,0,sizeof(UPARSE));
	up->next = ctx->parsefreelist;
	ctx->parsefreelist = up;
}

UPARSE *	/* or null on error */
UrlParse(ctx,url)
UrlContext *ctx;
char *url;	/* full url */
{
	UPARSE *up;
	char *p;
	char *uu;
	int need_leading_slash=0;
	int trailing_slash;
	int l;

	up = UrlNewParse(ctx);
	if (!up)
		return (UPARSE *)0;
	uu = UrlStrdup(url);	/* a copy for us to put nulls into */
	if (!uu) {
		UrlSetErrno(ctx,ENOMEM);
		UrlFreeParse(up);
		return (UPARSE *)0;
	}
	up->f1 = uu;			/* so we can free it later */
	up->url = UrlStrdup(uu);	/* a copy for us to save */
	if (uu[0]!='/') {
		p = strchr(uu,':');	/* look for access */
		if (p) {
			/* make sure the : is not after a / or # */
			char *pp;
			pp = strchr(uu,'/');
			if (!pp || pp>p) {
				pp = strchr(uu,'#');
				if (!pp || pp>p) {
					*p = 0;
					up->access = uu;
					uu = p+1;
				}
			}
		}
	}
	if (strncmp(uu,"//",2)==0) {	/* look for hostname */
		char *px;
		uu += 2;
		up->host = uu;
		p = strchr(uu,':');
		px = strchr(uu,'/');
		if (p && (!px || px>p)) {
			*p = 0;
			up->port = p+1;
			uu = p+1;
		}
		p = strchr(uu,'/');
		if (p) {
			*p = 0;
			uu = p+1;
			up->file = uu;
			need_leading_slash = 1;
		}
	} else {
		up->file = uu;
	}
	p = strchr(uu,'#');
	if (p) {
		uu = p+1;
		*p = 0;
		up->anchor = p+1;
	}
	p = strchr(uu,'?');
	if (p) {
		uu = p+1;
		*p = 0;
		up->query = p+1;
	}
	if (!up->file)
		up->file = "";
	l = strlen(up->file);
	trailing_slash = (l>0 && up->file[l-1]=='/');
		/* If the filename does not end in slash, then we add space
		 * to allow the app to add a trailing slash to the filename
		 * if it figures out that it's really a directory. */
	if (need_leading_slash || !trailing_slash) {
		/* malloc enough space for leading and trailing slashes */
		l = strlen(up->file)+3;
		p = UrlMalloc(l);
		if (!p) {
			UrlSetErrno(ctx,ENOMEM);
			UrlFreeParse(up);
			return (UPARSE *)0;
		}
		if (need_leading_slash) {
			p[0] = '/';
			strcpy(p+1,up->file);
		} else {
			strcpy(p,up->file);
		}
		up->f2 = p;
		up->file = p;
	}
	return up;
}

char *	/* belongs to us, we save in parse->unparsed field */
UrlUnparse(parse)
UPARSE *parse;
{
	int l;
	char *s, *p;
	UACCESS *ut;
	char *default_port;

	l = 5;	/* slop */
	if (parse->unparsed)
		UrlFree(parse->unparsed);
	if (parse->access) l+=strlen(parse->access)+1;
	if (parse->host) l+=strlen(parse->host)+2;
	if (parse->port) l+=strlen(parse->port)+1;
	if (parse->file) l+=strlen(parse->file)+1;
	if (parse->anchor) l+=strlen(parse->anchor)+1;
	if (parse->query) l+=strlen(parse->query)+1;
	s = UrlMalloc(l);
	if (!s) {
		UrlSetErrno(parse->ctx,ENOMEM);
		return (char *)0;
	}
	p = s;
	if (parse->access) {
		ut = UrlFindAccess(parse->ctx,parse->access);
		if (ut)
			default_port = ut->default_port;
		strcpy(p,parse->access);
		strcat(p,":");
		p += strlen(p);
	}
	if (parse->host) {
		strcpy(p,"//");
		strcat(p,parse->host);
		p += strlen(p);
	}
	if (parse->port &&
	    (!default_port || strcmp(default_port,parse->port)!=0)) {
		strcpy(p,":");
		strcat(p,parse->port);
		p += strlen(p);
	}
	if (parse->host && parse->file && parse->file[0]!='/') {
		*p++ = '/';
	}
	if (parse->file) {
		strcpy(p,parse->file);
		p += strlen(p);
	}
	if (parse->anchor) {
		strcpy(p,"#");
		strcat(p,parse->anchor);
		p += strlen(p);
	}
	if (parse->query) {
		strcpy(p,"?");
		strcat(p,parse->query);
		p += strlen(p);
	}
	parse->unparsed = s;
	return s;
}

/* Unparse if not already unparsed; return pointer to unparsed */
char *	/* belongs to us, we save in parse->unparse field */
UrlUnparseC(parse)
UPARSE *parse;
{
	if (!parse->unparsed)
		(void)UrlUnparse(parse);
	return parse->unparsed;
}

/* Check to see if the file ends in '/', add one if not.
 * This function relies on the fact that UrlParse guarantees to leave
 * extra space in the file string for that slash if needed. */
void
UrlParseMakeTrailingSlash(parse)
UPARSE *parse;
{
	int l;

	l = strlen(parse->file);
	if (parse->file[l-1]!='/') {
		parse->file[l]='/';	/* add trailing slash */
		parse->file[l+1]=0;
	}
}

/* end */
