/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlrel.c - resolve relative URLs
 *
 * Jim McBeath, October 23, 1994
 */

#include "urlopen.h"

extern char *strchr();
extern char *strrchr();

extern UPARSE *UrlNewParse UARGS((UrlContext *ctx));
extern char *UrlUnparse UARGS((UPARSE *parse));

/* count the number of chars c in s */
static int
strccount(s,c)
char *s;
int c;
{
	int n;

	n=0;
	while (s) {
		s = strchr(s,c);
		if (s) {
			n++;
			s++;
		}
	}
	return n;
}

/* find Nth char c in string s; N==1 does same thing as strchr */
static char *
strnchr(s,c,n)
char *s;
char c;
int n;
{
	char *p;

	if (n==0)
		return s;
	p = s-1;
	while (n>0) {
		p = strchr(p+1,c);
		if (!p)
			return p;
		n--;
	}
	return p;
}

/* remove n characters from s, pull the rest in */
static
void
strndel(s,n)
char *s;
int n;
{
	char *p;
	p = s;
	do {p[0]=p[n];} while (*p++);
}

char *		/* full URL */
UrlResolveRelative(ctx,base,partial)
UrlContext *ctx;
char *base;		/* fully-resolved URL */
char *partial;		/* piece of a url */
{
	UPARSE *ref;
	char *s;

	ref = UrlParse(ctx,base);
	if (!ref)
		return (char *)0;
	s = UrlResolveRelativeP(ctx,ref,partial);
	UrlFreeParse(ref);
	return s;
}

char *		/* full URL */
UrlResolveRelativeP(ctx,ref,partial)
UrlContext *ctx;
UPARSE *ref;		/* fully-resolved URL */
char *partial;		/* piece of a url */
{
	UPARSE *part;
	UPARSE full;
	char *s=0;
	char *p;
	int n;
	char *fullfile=0;
	static char *lastreturn=0;

	part = UrlParse(ctx,partial);
	if (!part)
		return (char *)0;
	if (lastreturn) {
		UrlFree(lastreturn);
		lastreturn = (char *)0;
	}
	full = *part;		/* copy all fields */
	if (full.access && *full.access) {
		/* access specified, no common ground between partial and ref */
		goto gotFull;
	}
	full.access = ref->access;
	/* TBD - default port numbers? */
	if ((part->host && *part->host) || (part->port && *part->port)) {
		/* host/port specified, don't try matching rest */
		goto gotFull;
	}
	full.host = ref->host;
	full.port = ref->port;
	if (!full.file || !*full.file) {
		full.file = ref->file;	/* no full, use base */
	} else if (full.file[0]!='/') {	/* relative path */
		n = strlen(full.file)+strlen(ref->file)+3;
		fullfile = UrlMalloc(n);
		if (!fullfile)
			goto memErr;
		strcpy(fullfile,ref->file);
		p = strrchr(fullfile,'/');
		if (p)
			p[1] = 0;	/* delete final component of base */
		else
			strcpy(fullfile,"./");	/* base is relative */
		strcat(fullfile,full.file);
		while (fullfile[0]=='.' && fullfile[1]=='/') {
			strndel(fullfile,2);
			while (fullfile[0]=='/')
				strndel(fullfile,1);
		}
		for (s=fullfile; *s; s++) {
			while (s[0]=='/'&&s[1]=='/'){
				/* remove extra slashes from path */
				strndel(s,1);
			}
			while (s[0]=='/'&&s[1]=='.'&& (s[2]=='/'||s[2]==0)) {
				/* remove "." from path */
				strndel(s,2);
			}
			while (s[0]=='/' &&s [1]=='.' && s[2]=='.'&&
			       (s[3]=='/'||s[3]==0) && s>fullfile) {
				/* remove ".." and preceding dir from path */
				for (p=s-1; p>fullfile && *p!='/'; p--)
					;	/* EMPTY - scan back to '/' */
				if (*p=='/')
					p++;
				strndel(p,s-p+3+(s[3]!=0));
				s = p-1;	/* s=p after s++ in for loop */
			}
		}
		full.file = fullfile;
	}
	/* leave anchor alone (never use anchor from ref) */
gotFull:
	s = UrlUnparse(&full);
	full.unparsed = (char *)0;	/* make it ours */
memErr:
	UrlFreeParse(part);
	if (fullfile)
		UrlFree(fullfile);
	lastreturn = s;	/* save so we can deallocate next time */
	return s;
}

/* end */
