/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlsym.c - generic symbol table management for url package
 *
 * Jim McBeath, October 27, 1994
 */

#include "urlopen.h"

extern char *strchr();

UrlSymTab *
UrlSymTabInit(ctx,name)
UrlContext *ctx;
char *name;	/* name of symbol table */
{
	UrlSymTab *tb;

	tb = (UrlSymTab *)UrlMalloc(sizeof(UrlSymTab));
	if (!tb) {
		UrlSetErrno(ctx,ENOMEM);
		return (UrlSymTab *)0;
	}
	memset(tb,0,sizeof(UrlSymTab));
	tb->ctx = ctx;
	tb->name = UrlStrdup(name);
	if (!tb->name) {
		UrlSetErrno(ctx,ENOMEM);
		UrlFree(tb);
		return (UrlSymTab *)0;
	}
	return tb;
}

/* free the table and all symbols in it */
void
UrlSymTabFree(ctx,table)
UrlContext *ctx;
UrlSymTab *table;
{
	UrlSym *usym, *usymnext;

	for (usym=table->first; usym; usym=usymnext) {
		usymnext = usym->next;
		UrlFree(usym->name);
		UrlFree(usym->value);
		UrlFree(usym);
	}
	UrlFree(table->name);
	UrlFree(table);
}

static
UrlSym *
UrlSymFindI(table,name)
UrlSymTab *table;
char *name;
{
	UrlSym *usym;

	for (usym=table->first; usym; usym=usym->next) {
		if (strcmp(name,usym->name)==0)
			return usym;
	}
	return (UrlSym *)0;
}

char *
UrlSymFind(ctx,table,name)
UrlContext *ctx;
UrlSymTab *table;
char *name;
{
	UrlSym *usym;

	usym = UrlSymFindI(table,name);
	if (usym)
		return usym->value;
	else
		return (char *)0;
}

/* Look for named symbol; if not found, see if name is of the form "a/b"
 * and look for "a/*" */
char *
UrlSymFindSlashStar(ctx,table,name)
UrlContext *ctx;
UrlSymTab *table;
char *name;
{
	char *s;
	char *p;
	char *nn;

	s = UrlSymFind(ctx,table,name);
	if (s)
		return s;
	p = strchr(name,'/');
	if (!p)
		return (char *)0;
	nn = UrlStrdup(name);
	if (!nn)
		return (char *)0;
	p = nn + (p-name) + 1;
	p[0] = '*';
	p[1] = 0;
	s = UrlSymFind(ctx,table,p);
	UrlFree(nn);
	return s;
}

int	/* 0 if OK */
UrlSymSet(ctx,table,name,value)
UrlContext *ctx;
UrlSymTab *table;
char *name;
char *value;
{
	UrlSym *usym;
	char *cc, *ee;

	usym = UrlSymFindI(table,name);
	cc = UrlStrdup(value);
	if (!cc) {
		UrlSetErrno(ctx,ENOMEM);
		return ENOMEM;
	}
	if (usym) {
		UrlFree(usym->value);
		usym->value = cc;
	} else {
		usym = (UrlSym *)UrlMalloc(sizeof(UrlSym));
		if (!usym) {
			UrlFree(cc);
			UrlSetErrno(ctx,ENOMEM);
			return ENOMEM;
		}
		ee = UrlStrdup(name);
		if (!ee) {
			UrlFree(cc);
			UrlFree(usym);
			UrlSetErrno(ctx,ENOMEM);
			return ENOMEM;
		}
		usym->name = ee;
		usym->value = cc;
		usym->next = table->first;
		table->first = usym;
	}
	return 0;
}

/* end */
