/* Copyright 1995 Jim McBeath
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that my name not be used in advertising
 * or publicity pertaining to distribution of the software without
 * specific, written prior permission.  I make no representations about
 * the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * I disclaim all warranties with regard to this software, including all
 * implied warranties of merchantability and fitness.  In no event shall
 * I be liable for any special, indirect or consequential damages or any
 * damages whatsoever resulting from loss of use, data or profits, whether
 * in an action of contract, negligence or other tortious action, arising
 * out of or in connection with the use or performance of this software.
 */
/* urlutil.c - misc utility functions
 *
 * Jim McBeath, October 23, 1994
 */

#include "urlopen.h"

extern char *strchr();
extern char *strrchr();

extern int UrlSigPipeFlag;

int			/* number of fields found */
UrlSplitFields(s,c,m,pp,np)
char *s;		/* the string to split - puts nulls into it in place */
int c;			/* the char to split on */
int m;			/* true means ignore multiple occurances of c */
char **pp;		/* RETURN - array to fill with pointers */
int np;			/* size of pp array */
{
	int n;
	char *e;

	if (!s)
		return 0;
	n = 0;
	do {
		*pp++ = s;
		n++;
		if (n>=np)
			break;
		e = strchr(s,c);
		if (!e)
			break;
		*e++ = 0;
		if (m) {
			while (*e==c)
				e++;
		}
		s = e;
	} while (*s);
	return n;
}

char *
UrlDirFileHtml(uf,fileinfo)
UFILE *uf;
UrlDirFileInfo *fileinfo;
{
#define DIRFILEHTMLBUFSIZE 1200
	static char buf[DIRFILEHTMLBUFSIZE];
	char sizestr[30], sizestr2[30];
	char *name;
	char *anchor;
	char *ext;
	char *cont;
	char *icon;
	char *icon_prefix;

	if (!fileinfo->name)
		return (char *)0;
	if (fileinfo->size==0 && fileinfo->isdir)
		sizestr[0] = 0;		/* blank string */
	else if (fileinfo->size<=9999)
		sprintf(sizestr,"%d",fileinfo->size);
	else if (fileinfo->size/1000<=9999)
		sprintf(sizestr,"%dK",fileinfo->size/1000);
	else
		sprintf(sizestr,"%dM",fileinfo->size/1000000);
	if (fileinfo->symlink && sizestr[0]) {
		/* if symlink, put size in parens to indicate that to user */
		strcpy(sizestr2,sizestr);
		sprintf(sizestr,"(%s)",sizestr2);
	}
	ext = strrchr(fileinfo->name,'.');
	if (ext)
		ext++;
	else
		ext = "";
	if (fileinfo->isdir)
		cont = "application/directory";
	else if (fileinfo->isexec)
		cont = "application/executable";
	else {
		cont = UrlGetExtContType(uf->ctx,ext);
		if (!cont)
			cont = "unknown";
	}
	icon = UrlGetContIconType(uf->ctx,cont);
	if (!icon)
		icon = "unknown";
	name = fileinfo->name;
	if (fileinfo->symlink)
		anchor = fileinfo->symlink;
	else
		anchor = fileinfo->name;
	anchor = UrlResolveRelativeP(uf->ctx,uf->parse,anchor);
	icon_prefix = uf->ctx->icon_prefix;
	if (!icon_prefix)
		icon_prefix = "internal:icons/";
	sprintf(buf,"\
<img src=\"%s%s\">  %-8s %-15s <a href=\"%s\">%s</a>\n",
		icon_prefix, icon, sizestr, fileinfo->date, anchor, name);
	return buf;
}

/* buffer belongs to us, will be overwritten on next call */
char *UrlGetFileString(uf)
UFILE *uf;
{
	int t;
	static char *buf=0;
	static int bufsize=0;
	char *ebuf, *p, *s;
	FILE *f;
	int slen;

#define READSIZE 250
	if (bufsize<READSIZE+1) {
		if (buf) UrlFree(buf);
		bufsize = READSIZE+1;
		buf = UrlMalloc(bufsize);
	}
	p = buf;
	*p = 0;
	ebuf = buf+bufsize;
	if (UrlIsFiltered(uf)) {
		s = "";
		while (s) {
			s = UrlGetLine(uf);
			if (s) {
				slen = strlen(s);
				if (ebuf-p<slen+1) {
					t = p-buf;
					while (bufsize<ebuf-buf+slen+1)
						bufsize *= 2;
					buf = UrlRealloc(buf,bufsize);
					p = buf+t;
					ebuf = buf+bufsize;
				}
				strcpy(p,s);
				p += slen;
			}
		}
	} else {
		while (!UrlUeof(uf)) {
			if (ebuf-p<READSIZE+1) {
				t = p-buf;
				bufsize *= 2;
				buf = UrlRealloc(buf,bufsize);
				p = buf+t;
				ebuf = buf+bufsize;
			}
			t = UrlUread(p,1,READSIZE,uf);
			p[t] = 0;
			if (t==0) break;
			p += t;
		}
	}
	return buf;
}

int	/* same as unix write */
UrlFWrite(fd,buf,size)
FILE *fd;
char *buf;
int size;
{
	int t;

	UrlSigPipeFlag = 0;
	t = write(fd,buf,size);
	if (UrlSigPipeFlag)
		return UrlSigPipeFlag;
	return t;
}

/* end */
